
/*
 * Klima Fernueberwachung via email.
 * (c)2021  Peter Klingbeil Version 2
 * -- Initiale Version  --
 * Fuer das ESP32 DEV Module
 * 05-08-22 Peter Klingbeil Version 3 
 * Neue DHT Bibliothek (DHT11 & DHT22),
 * GMT Zeit eingefuegt, 
 * Messinterval geaendert, 
 * Fehler Behandlung
 * 23-11-23 Peter Klingbeil Version 4
 * Korrektur: kein Rueckgabewert mehr bei test()
 * Neue Funktion: KeinFehler()
 * 
 * Mail Software:
 * originally Created by K. Suwatchai (Mobizt)
 * Email: k_suwatchai@hotmail.com
 * Github: https://github.com/mobizt
 * Copyright (c) 2019 mobizt
 * 
 * Sensor Bibliothek
 * https://create.arduino.cc/projecthub/mafzal/temperature-monitoring-with-dht22-arduino-15b013  
 * 
*/

//To use send Email for Gmail to port 465 (SSL), less secure app option should be enabled. https://myaccount.google.com/lesssecureapps?pli=1
//To receive Email for Gmail, IMAP option should be enabled. https://support.google.com/mail/answer/7126229?hl=en

int DigIn=12;        // Digitaleingang 12 
int DigOut13=13;     // Digitalausgang 13
int DigOut14=14;     // Digitalausgang 14 

bool x12=true;       // Entprellung Taste
volatile int InterruptDone = 0;   // Interupt Flag
volatile unsigned long alteZeit=0, entprellZeit=500;  // Taste entprellen 500 ms    

#include "ESP32_MailClient.h" 
#include "SD.h"

//For demo only 
#include "image.h"

//TEMP AND HUMIDITY Sensor DHT22 
#include "DHT.h"
float Luftfeuchtigkeit = 0;
float Temperatur = 0;

#define WIFI_SSID "Ihre SSD"
#define WIFI_PASSWORD "Ihr Passwort"
#define SD_CS 5

// Set email Data and smtp Server access
#define emailSenderAccount    "test@test.de"    // My Mail Address
#define emailSenderPassword   "test"            // My Mail PW
#define emailRecipient        "mail@test.de"    // E-Mail Receipt
#define smtpServer            "test.de"         // Outgoing Server
#define smtpServerPort        587               // Port
#define emailSubject          "ESP32"           // Subject to send

//      for DHT22: 
//      VCC: 5V or 3.3V
//      GND: GND
//      DATA: 2

#define DHTPIN 2        //Der Sensor wird an PIN 2 angeschlossen    
#define DHTTYPE DHT22   // Es handelt sich um den DHT22 Sensor
DHT dht(DHTPIN, DHTTYPE); //Der DHT22 Sensor

unsigned int Counter=0;
volatile int MailSend = 0;  // Mail OK Flag

//The Email Sending data object contains config and data to send
SMTPData smtpData;   //Email

//Callback function to get the Email sending status
void sendCallback(SendStatus info);

byte temperature = 0;  // Sensor Data
byte humidity = 0;
byte data[40] = {0};

struct tm timeinfo;      // Zeit Array
String dataString = "";  // fuer die LogDatei

void setup()
{

  Serial.begin(115200);
  Serial.println();

  pinMode(DigIn,INPUT_PULLUP); // Pin 12 = Taste
  pinMode(DigOut13,OUTPUT);    // LED Rot
  pinMode(DigOut14,OUTPUT);    // LED Gruen
  
  digitalWrite(DigOut13, HIGH); // Status LED Rot ein
  digitalWrite(DigOut14, LOW); // Status LED Gruen aus
  
  Serial.println("Logger Version 4.0");
  
  Serial.print("Connecting to AP");
  
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  while (WiFi.status() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(200);
  }

  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println();

  attachInterrupt(digitalPinToInterrupt(DigIn), FTASTE, CHANGE);  // Interrupt zuweisen: Taste
  
  digitalWrite(DigOut13, LOW); // Status LED Rot aus
  digitalWrite(DigOut14, HIGH); // Status LED Gruen ein

  Serial.println("Mounting SD Card...");

  if (SD.begin(SD_CS)) // Test SD-Card
  {
    Serial.println("SD-Card OK");
   }
  else
  {
    Serial.println("SD Card Monting Failed");
    Fehler();
  }

  Serial.println();
  
  dht.begin(); //DHT Sensor starten
  setClock();  // Uhrzeit aus dem Internet holen

}

void FTASTE()   // Interrupt & Entprellung der Taste
{
   if((millis() - alteZeit) > entprellZeit) 
   { 
    // innerhalb der entprellZeit nichts machen
    InterruptDone=1;
    alteZeit = millis(); // letzte Schaltzeit merken      
  }  
}


void setClock() // Uhrzeit aus dem Internet holen
 {
  configTime(0, 0, "pool.ntp.org");

  //Serial.print(F("Waiting for NTP time sync: "));
  time_t nowSecs = time(nullptr);

  while (nowSecs < 8 * 3600 * 2) 
   {
    delay(500);
    Serial.print(F("."));
    yield();
    nowSecs = time(nullptr);
   }
   

  //struct tm timeinfo;
  gmtime_r(&nowSecs, &timeinfo);  //GMT Zeit
  Serial.print(F("Get current GMT time: "));
  Serial.println(asctime(&timeinfo));
  //Serial.print(timeinfo.tm_hour);
  //Serial.print(":");
  //Serial.print(timeinfo.tm_min);
  //Serial.print(":");
  //Serial.println(timeinfo.tm_sec);
 }


void SendEmail()
{
  Serial.println("Sending email...");
  MailSend = 0;
  
  // Set the SMTP Server Email host, port, account and password
  smtpData.setLogin(smtpServer, smtpServerPort, emailSenderAccount, emailSenderPassword);
  //For library version 1.2.0 and later which STARTTLS protocol was supported,the STARTTLS will be 
  //enabled automatically when port 587 was used, or enable it manually using setSTARTTLS function.
  //smtpData.setSTARTTLS(true);

  // Set the sender name and Email
  smtpData.setSender("ESP32", emailSenderAccount);

  // Set Email priority or importance High, Normal, Low or 1 to 5 (1 is highest)
  smtpData.setPriority("High");

  // Set the subject
  smtpData.setSubject(emailSubject);
  
  // Set the email message in text format (raw)
  smtpData.setMessage("Klima Report - Sent from ESP32 board", false);

  // Add recipients, you can add more than one recipient
  smtpData.addRecipient(emailRecipient);
  //smtpData.addRecipient("YOUR_OTHER_RECIPIENT_EMAIL_ADDRESS@EXAMPLE.com");
  smtpData.setSendCallback(sendCallback);
  

  //Add attachments, can add the file or binary data from flash memory, file in SD card
  //Data from internal memory
  //smtpData.addAttachData("firebase_logo.png", "image/png", (uint8_t *)dummyImageData, sizeof dummyImageData);

  //Add attach files from SD card
  //Comment these two lines, if no SD card connected
  //smtpData.addAttachFile("/Bild.jpg");
  smtpData.addAttachFile("/Klima.txt");


  //Add some custom header to message
  //See https://tools.ietf.org/html/rfc822
  //These header fields can be read from raw or source of message when it received)
 
  //smtpData.addCustomMessageHeader("Date: Sat, 19 Jan 2021 10:39:56 -0700 (PDT)");
  
  //Be careful when set Message-ID, it should be unique, otherwise message will not store
  //smtpData.addCustomMessageHeader("Message-ID: <abcde.fghij@gmail.com>");

  //Set the storage types to read the attach files (SD is default)
  //smtpData.setFileStorageType(MailClientStorageType::SPIFFS);

  smtpData.setSendCallback(sendCallback);

  //Start sending Email, can be set callback function to track the status
  if (!MailClient.sendMail(smtpData))
  {
    Serial.println("Error sending Email, " + MailClient.smtpErrorReason());
    Fehler();
    MailSend = 1;
  }
  else
  {
    KeinFehler();
  }

  //Clear all data from Email object to free memory
  smtpData.empty();
}

void Fehler()  // Sammelmeldung
{
  digitalWrite(DigOut13, HIGH); // Status LED Rot ein
  digitalWrite(DigOut14, LOW); // Status LED Gruen aus 
}

void KeinFehler()  // Sammelmeldung ruecksetzen
{
  digitalWrite(DigOut13, LOW); // Status LED Rot aus
  digitalWrite(DigOut14, HIGH); // Status LED Gruen ein 
}


void test()   // DHT Sensordaten auslesen
{
  InterruptDone=0;
  delay(1000);
  File file = SD.open("/Klima.txt", FILE_WRITE);
  
  if(file)
   {
    file.print("Startzeit: ");
    dataString=(asctime(&timeinfo));  // Datum an den Anfang der neuen Log Datei
    file.println(dataString);
    dataString="";
    
    while(InterruptDone==0)
     {
     // Sensor
     delay(1000);
     
     Luftfeuchtigkeit = dht.readHumidity();
     Temperatur = dht.readTemperature();
          
     if (isnan(Luftfeuchtigkeit))
     {
       Serial.println("Read DHT-Sensor failed");
       Fehler();
     }
     else
     {
      Serial.println("Read DHT-Sensor OKay");
      KeinFehler();
     }

     //Luftfeuchtigkeit=Luftfeuchtigkeit - 10; // Korrektur DHT22 11.2023
     
     //Lineare Lorrektur fuer DHT11 Sensor
     //Luftfeuchtigkeit=Luftfeuchtigkeit-21; // Sensor Korrektur
     //Temperatur=Temperatur+2,2; // Sensor Korrektur

     // Write to file
       setClock(); // Uhrzeit holen
       dataString += String(timeinfo.tm_hour);
       dataString += String(":");
       dataString += String(timeinfo.tm_min);
       dataString += String(";");
      
       file.print(String(dataString));
       file.print(Temperatur);
       file.print(";");
       file.println(Luftfeuchtigkeit);
          
       Serial.print(String(dataString));
       Serial.print(" ");
       Serial.print(Temperatur); Serial.print(" C, ");
       Serial.print(Luftfeuchtigkeit); Serial.print(" %");
       Serial.print(" Count: "); Serial.println(Counter);

       dataString = ""; //Puffer löschen
           
       if(++Counter > 300)  // 300 mal 1 Min. = 5 Stunden
        {
         InterruptDone = 1;
         return;    // Send E-Mail
        }                  
        delay(60000); //Messzyklus 1 Minute 
    } 
  }   
}


//Callback function to get the Email sending status
void sendCallback(SendStatus msg)
{
  //Print the current status
  Serial.println(msg.info());

  //Do something when complete
  if (msg.success())
  {
    Serial.println("- Email Send succesfull-");
  }
}

void loop()
{
  test();  // Messung
  
  if (InterruptDone == 1) // Eingabe: ist die Auswahltaste betaetigt?
  
  {
   Serial.println("Interrupt! ");
   delay(100); 
   SendEmail();    // Send E-Mail
   delay(2000);
   Counter=0;
   
   if (MailSend == 0) // Nur loeschen wenn Email send OK
    {
     SD.remove("Klima.txt");    // File loeschen
     delay(300);
     File file = SD.open("Klima.txt", FILE_WRITE);
    }
    
   digitalWrite(DigOut13, LOW); // Status LED Rot aus
   digitalWrite(DigOut14, HIGH); // Status LED Gruen ein
   InterruptDone = 0;
  }
}
